///////////////////////////////////////////////////////////////////////////////
// Tyler Perkins
// 9-23-21
// Forth compiler/interpreter
// entry point
//

#include <stdio.h>
#include <string.h>

#include "parse.h"
#include "stack.h"

#define BUFFER_SIZE 1024

void help(int argc, char** argv){
    printf("Incorrect usage!\n");
    printf("Usage:\n");
    printf("  %s [FILE]\n", argv[0]);
    printf("File is an optional forth file containing code\n");
    printf("If no file is provided you will be dropped into an interactive shell\n");
    printf("All operators are using reverse Polish notation\n");
    printf("Max line input size is %i", BUFFER_SIZE);
}

void process_file(FILE* in, char* input){
    char* fg;
    int retval;
    while(!(fg = fgets(input, BUFFER_SIZE, in)) || feof(in)){
        retval = parse(input);
        if(retval > -1){
            printf("%s\n ",input);
            for(int i = 1; i != retval; ++i){
                printf(" ");
            }
            printf("^\n");
            printf("Error parsing character at position %d!\n", retval);
            break;
        }
    }
}

int main(int argc, char** argv){
    char input[BUFFER_SIZE] = {0};
    int retval = 0;
    FILE* in = stdin;
    char* fg;

    //check arg count
    if(argc > 2){
        help(argc, argv);
        return -1;
    } else if(argc == 2){
        in = fopen(argv[1], "r");
        if(in == NULL){
            printf("Couldn't open file %s\n", argv[1]);
            in = stdin;
        }
    }

    if(in != stdin){
        process_file(in, input);
    }

    printf("Entering shell...\n");

    //get user input, with prompt
    printf("> ");
    fg = fgets(input, BUFFER_SIZE, in);

    while(strcmp(input, "exit\n")){
        if(fg == NULL) //ctrl+d
            break;


        retval = parse(input);
        if(retval > -1){
            printf("  ");
            for(int i = 1; i != retval; ++i){
                printf(" ");
            }
            printf("^\n");
            printf("Error parsing character at position %d!\n", retval);
        } else {
            printf("%d\n", peek());
        }

        printf("> ");
        fg = fgets(input, BUFFER_SIZE, in);
    }

    printf("Goodbye\n");
    return 0;
}
